"use strict";
this.name           = "SaveInFlight";
this.author         = "Dybal";
this.copyright      = "2022 Dybal";
this.license        = "CC-BY-NC-SA 4.0";
this.description    = "Allows saving the game in flight";
this.version        = "1.0.0";

this.$debug = false;
this.$logging = true;
this.$sifnabled = false;
this.$position; // player ship saved position
this.$orientation; // player ship saver orientation
this.$sif_station = null; // Station at the End of Universe :-)

//
// Event Handlers
//

//------------------------------------------------------------------------------------------//
this.startUpComplete = function _SaveInFlight_startUpComplete() {
    this.ship = player.ship;
    if (this.ship.equipmentStatus("EQ_SAVE_IN_FLIGHT") === "EQUIPMENT_OK")
        this.$sifEnabled = true;
    else
        this.$sifEnabled = false;
    if (missionVariables.SaveInFlight_position) {
        var pos = JSON.parse(missionVariables.SaveInFlight_position);
        var or = JSON.parse(missionVariables.SaveInFlight_orientation);
        this.$position = new Vector3D(pos)
        this.$orientation = new Quaternion(or);
        log(this.name, "missionVariables, position: "+pos+", orientation: "+or);
        log(this.name, "Ship position: "+this.$position+", orientation: "+this.$orientation);
        delete missionVariables.SaveInFlight_position;
        delete missionVariables.SaveInFlight_orientation;
    }
    if (worldScripts.BGS) {
        //this.$bg = getScreenBackgroundForKey("shiplibrary");
        this.$bg = {name:"bgs_status_void.png"};
    } else { 
        this.$bg = {name:"splash@2x.png"};
    }
}

//------------------------------------------------------------------------------------------//
this.systemWillPopulate = function _SaveInFlight_systemWillPopulate() {
    log(this.name, "SystemWillPopulate");
    system.setPopulator("End of Universe Station", {
        callback: function(pos) {
            var ws = worldScripts.SaveInFlight;
            var s = system.addShips("sif_station", 1, pos, 0);
            if (s && s.length == 1) {
                ws.$sif_station = s[0];
                ws.$sif_station.displayName = "Station at the End of Universe";
                ws.$sif_station.breakPattern = false;
                log(ws.name, "Created station "+ws.$sif_station);
            } else {
                log(ws.name, "Could not create the Station at the End of Universe");
            }
        }.bind(this),
        location: "COORDINATES",
        coordinates: Vector3D(0, 0, -1E4).fromCoordinateSystem("spu"),
        deterministic: true
    });
}

//------------------------------------------------------------------------------------------//
this.equipmentAdded = function _SaveInFlight_equipmentAdded(equipmentKey) {
    if (equipmentKey === "EQ_SAVE_IN_FLIGHT") {
        if (this.$debug) log(this.name, "Save in Flight added")
        this.$sifEnabled = true;
    }
}

//------------------------------------------------------------------------------------------//
this.equipmentRemoved = function _SaveInFlight_equipmentRemoved(equipmentKey) {
    if (equipmentKey === "EQ_SAVE_IN_FLIGHT") {
        if (this.$debug) log(this.name, "Save in Flight removed")
        this.$sifEnabled = false;
    }
}

//------------------------------------------------------------------------------------------//
this.playerWillSaveGame = function _SaveInFlight_playerWillSaveGame() {
    var pos = this.$position;
    var or = this.$orientation;
    if (this.$position) {
        missionVariables.SaveInFlight_position = JSON.stringify([pos.x, pos.y, pos.z]);
        missionVariables.SaveInFlight_orientation = JSON.stringify([or.w, or.x, or.y, or.z]);
    }
}

//------------------------------------------------------------------------------------------//
this.shipLaunchedFromStation = function _SaveInFlight_shipLaunchedFromStation(station)  {
    var p = player.ship;
    if (station == this.$sif_station) {
        log(this.name, "Placing player ship at "+this.$position+" with orientation "+this.$orientation+", speed:" +p.speed);
        var max_speed = p.maxSpeed;
        p.maxSpeed = 0;
        p.position = this.$position;
        p.orientation = this.$orientation;
        p.maxSpeed = max_speed;
        this.$position = this.$orientation = null;
    }
}

//------------------------------------------------------------------------------------------//
this.guiScreenChanged = function _SaveInFlight_guiScreenChanged(to, from) {
    var pship = player.ship;
    if (!pship.docked || !pship.dockedStation.hasRole("sif_station")) return;
    if (guiScreen === "GUI_SCREEN_EQUIP_SHIP") {
        mission.runScreen({
            title: "Station at the End of Universe",
            message: "This station has no Shipyard",
            background: this.$bg 
        });
    } else if (guiScreen === "GUI_SCREEN_MARKET") {
        mission.runScreen({
            title: "Station at the End of Universe",
            message: "This station has no Market",
            background: this.$bg 
        });
    } else if (guiScreen === "GUI_SCREEN_INTERFACES") {
        mission.runScreen({
            title: "Station at the End of Universe",
            message: "This station has no Interfaces",
            background: this.$bg 
        });
    }     
}

//------------------------------------------------------------------------------------------//
this.activated = function _SaveInFlight_activated() {
    var ws = worldScripts.SaveInFlight;
    var p = player.ship;
    if (ws.$debug) log(ws.name, "Trying to activate Save In Flight: "+ws.$sifEnabled);
    if (ws.$sifEnabled) {
        if (p.status != "STATUS_IN_FLIGHT") {
            player.consoleMessage("Ship must be in flight for saving", 30);
            return;
        }
        var in_range = p.checkScanner();
        var n=0;
        for (var i=0;i<in_range.length; i++) {
            if (in_range[i].dataKey != "telescopemarker") n++;
        }
        if (n > 0) {
            if (ws.$debug) log(ws.name, "In range: "+in_range);
            player.consoleMessage("There must be nothing in scanner range for saving", 30);
            return;
        }
        if (p.speed > 0) {
            player.consoleMessage("Ship must be stopped for saving", 30);
            return;
        }
        ws.$position = p.position;
        ws.$orientation = p.orientation;
        if (ws.$debug) log(ws.name, "SaveInFlight activated, ship position: "+ws.$position+" and orientation: "+ws.$orientation);
        if (!ws.$sif_station.dockPlayer()) {
            log(ws.name, "Could not \"dock\" player at the Station at the End of Universe");
        }
    }
}
